#include <windows.h>
#include "perl.h"
#include "Pipe.h"
#include "PipeBuild.h"

// Icon
// Win32Perms   ICON    DISCARDABLE     "Perms.ICO"


VS_VERSION_INFO VERSIONINFO
    FILEVERSION        FILE_VERSION  //  PERLRC_VERSION
    PRODUCTVERSION     PROD_VERSION  //  PERLRC_VERSION
	FILEFLAGSMASK      VS_FFI_FILEFLAGSMASK
	FILEFLAGS          VER_DEBUG
	FILEOS             VOS_NT_WINDOWS32
	FILETYPE           VFT_DLL
	FILESUBTYPE        VFT2_UNKNOWN

BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", 			RC_COMMENTS
            VALUE "CompanyName", 		RC_COMPANY
            VALUE "FileDescription", 	RC_FILEDESC 
            VALUE "FileVersion", 		RC_FILEVER
            VALUE "InternalName", 		RC_INTNAME
            VALUE "LegalCopyright", 	RC_COPYRIGHT
            VALUE "OriginalFilename", 	RC_FILENAME
            VALUE "PrivateBuild", 		RC_PBUILD
            VALUE "ProductName", 		RC_PRODNAME
            VALUE "ProductVersion", 	RC_PRODVER
        END
    END
	BLOCK "VarFileInfo"
    BEGIN
       	VALUE "Translation", 0x409, 1200
    END

	BLOCK "VarFileInfo"
	BEGIN
		VALUE "Translation", 0x409, 0x04E4
		// English language (0x409) and the Windows ANSI codepage (0x04E4)
	END
END
