#ifndef _VERSION_H_
    #define _VERSION_H_

    #define VERSION_STRING          "20010208"
    #define VERSION_HI              ( 0x7d1 )
    #define VERSION_LO              ( 0xd0 )

    #define FILE_VERSION            0x7d1, 0x02, 0x08, 0x00
    #define PROD_VERSION            0x7d1, 0x02, 0x08, 0x00

#endif 

